function root = createBranchedRoot(rootparameters, dx)
% createBranchedRoot: Creates several orders of a branched root
%
% The root parameters of each order of the branchend root are given by the
% following fields:
% r:            initial growth rate
% la, lb, ln:   length of apical, basal and inter-root distance.
% nob:          number of lateral branches
% k:            maximal length (for highest order roots only instead of la,
%     lb, ln, nob) 
% a:            radius
% theta:        angle between root and later root of next order 
% color:        color of the root
% N:            strength of gravitropism
% sigma:        additional random noise to the heading
% bend:         an arbitrary deflection rule function (instead of N and
%      sigma) 
%
% rs, las, lbs, lns, nobs, ... are the corresponding standard deviations of
% each root parameter.
%
% Parameters:
% rootparameters   :a list of root parameters of every order
% dx               :spatial discrtetization along the root. Either one
%       global value for all root orders, or a vector containing a value
%       for each order.
%
% root             :the resulting L-system letter
%
% See also: Example_Maize
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

noo = length(rootparameters); % number of orders

if length(dx)~=noo
    dx = ones(1,noo)*dx;
end

for i = 1 : noo

    j = noo-i+1;

    a = rootparameters{j}.a;
    as = rootparameters{j}.as;
    color = rootparameters{j}.color;
    r = rootparameters{j}.r;
    rs = rootparameters{j}.rs;

    if (j~=1) % force 0-th order root to have theta=0
        theta = rootparameters{j}.theta;
        thetas = rootparameters{j}.thetas;
    else
        if isfield(rootparameters{j},'theta')
            theta = rootparameters{j}.theta;
            thetas = rootparameters{j}.thetas;
        else
            theta=0;
            thetas=0;
        end
    end
        
    if (j==noo)
        if (isfield(rootparameters{j},'k'))
            k = rootparameters{j}.k;
            ks = rootparameters{j}.ks;
        else
            k = rootparameters{j}.la+rootparameters{j}.lb+rootparameters{j}.ln*rootparameters{j}.nob;
            ks = rootparameters{j}.las+rootparameters{j}.lbs+ rootparameters{j}.lns*rootparameters{j}.nobs;
        end
    else
        la = rootparameters{j}.la;
        las = rootparameters{j}.las;
        lb = rootparameters{j}.lb;
        lbs = rootparameters{j}.lbs;
        ln = rootparameters{j}.ln;
        lns = rootparameters{j}.lns;
        nob = rootparameters{j}.nob;
        nobs = rootparameters{j}.nobs;
    end

    if isfield(rootparameters{j},'bend')  % is there a user defined tropism
        deflect = rootparameters{j}.bend;
        %disp(rules.Bend);
    else
        sigma = rootparameters{j}.sigma;
        if isfield(rootparameters{j},'N')
            N = rootparameters{j}.N;
            deflect = @(x,dx) ruleGravitropismN(x,N,sigma,dx);
        else
            alpha = rootparameters{j}.alpha;
            if isfield(rootparameters{j},'exp')
                exp = rootparameters{j}.exp;
            else
                exp = 1;
            end
            deflect = @(x,dx) ruleGravitropism(x,[0,0,-1],alpha,exp,sigma,dx);
        end
    end
        
    if (j==noo)
        newbranch.name = 'G';
        if isfield(rootparameters{j},'lambda') % is there a user defined growth function
            newbranch.apply = @(x,dt) ruleCreateHighestOrderRoot(x,r,rs,k,ks,theta,thetas,...
                a,as,color,deflect,dx(j),dt,Inf,rootparameters{j}.lambda);
        else
            newbranch.apply = @(x,dt) ruleCreateHighestOrderRoot(x,r,rs,k,ks,theta,thetas,...
                a,as,color,deflect,dx(j),dt);
        end
    else
        newbranch.name = 'G';
        if isfield(rootparameters{j},'lambda') % is there a user defined growth function
            newbranch.apply = @(x,dt) ruleCreateRoot(x,r,rs,la,las,lb,lbs,ln,lns,nob,nobs,theta,thetas,...
                a,as,color,cbranch,deflect,dx(j),dt,Inf,rootparameters{j}.lambda);
        else
            newbranch.apply = @(x,dt) ruleCreateRoot(x,r,rs,la,las,lb,lbs,ln,lns,nob,nobs,theta,thetas,...
                a,as,color,cbranch,deflect,dx(j),dt);
        end
    end

    if isfield(rootparameters{j},'prebranch') % is there an additional rule to put in front of a branch
        clear newbranch2;
        newbranch2.name='G';
        f = rootparameters{j}.prebranch;
        newbranch2.apply= @(x,dt) f.apply(x,{newbranch},dt);
        cbranch = {newbranch2}; % current branch
    else
        cbranch = {newbranch}; % current branch
    end
        
end

root = cbranch;



