function [map,total] = getMap(str,xp,yp,zp,mode,wp,cp,tp)
% getMap: Creates an averaged map from an L-System string.

% The map contains either averaged volume, surface or length over the
% corresponding rectangular domains.
%
% Parameters:
% str               :L-System string
% xp                :partition of the x-axis
% yp                :partition of the y-axis
% zp                :partition of the z-axis
% (mode)            :'v' :volume, 's' :surface, 'l' :length; capital
%        letters to exclude dead root segments (with color=[1,1,1]).
%        (default mode='s').
% (wp)              :partition of diameters. default wp=[0.5].
% (cp)              :partition of (grey scaled) colors. default cp=[0.5].
% (tp)              :partition of time. Default = [0.5];
%
% map               :map of size (length(xp)-1)*(length(yp)-1)*
%          (length(zp)-1)
% total             :total volume, surface or length
%
% See also: Example_DensityMap
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

if (nargin<5)
    mode='s';
end

if (nargin<6)
    wp=0.5;
end

if (nargin<7)
    cp=0.5;
end

if (nargin<8)
    tp = 0.5;
end

map = zeros(length(xp),length(yp),length(zp),length(wp),length(cp));
total = 0;

%
% faster than getting the index via  k =
% interp1(zp,1:length(zp),zT(2),'nearest');
%
xp = getEdges(xp);
yp = getEdges(yp);
zp = getEdges(zp);
wp = getEdges(wp);
cp = getEdges(cp);
tp = getEdges(tp);

[x1,x2,r,color,time]=getSegments(str);
mid = (x1+x2)/2;
len = sqrt(sum((x2-x1).^2,2));

for ii=1 : length(x1)

    cT_ = sum(color(ii))/3;
    wT = 2*r(ii);
    tT = time(ii);

    switch mode
        case 'v'
            v = len(ii)*(wT/2)^2*pi; %volume;
        case 's'
            v = len(ii)*wT*pi; %surface;   %TODO !
        case 'l'
            v = len(ii); %length
        case 'V'
            if (cT_~=1)
                v = len(ii)*(wT/2)^2*pi; %volume if alive;
            else
                v=0;
            end
        case 'S'
            if (cT_~=1)
                v = len(ii)*wT*pi; %surface if alive;
            else
                v=0;
            end
        case 'L'
            if (cT_~=1)
                v = len(ii); %length if alive
            else
                v=0;
            end
    end

    i=sum(xp<mid(ii,1));
    j=sum(yp<mid(ii,2));
    k=sum(zp<mid(ii,3));
    l=sum(cp<cT_);
    m=sum(wp<wT);
    n=sum(tp<tT);

    map(i,j,k,l,m,n) = map(i,j,k,l,m,n) + v;
    total = total + v;

end


function y = getEdges(x)
y = ones(1,length(x)+1);
for i=1 :length(x)-1
    y(i+1) = (x(i)+x(i+1))/2;
end
y(1) = -Inf;
y(end)= Inf;







