function [lines,widths,colors,times,c] = getPolylines(str, initial)
% getPolyline: Interprets a L-System string graphically using Turtle graphic.
%
% The function returns a list of polylines, diameters, colors and segment
% creation times. Every poly line represents a single root axis. The result
% can be easily represented using plotTubes.
%
% Parameters:
% str             :List of L-System letters
% (initial)       :Initial values of the Turtle. Default: initial.xT=0,
%       initial.yT=0, initial.zT=0, initial.wT=0.1, initial.cT = [0,1,0],
%       R=diag([1,1,1])
%
% lines           :a list of polylines described by a vector of points
% width           :a list of segment diameters (at the end points).
% colors          :a list of colors (at the end points)
% times           :a list of segment creation times (at the end points)
% c               :total number of segments
%
% See also: plotTubes, getSegments.
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

if (nargin<2)
    xT = 0;
    yT = 0;
    zT = 0;
    wT = 0.1;
    cT = [0, 1, 0];
    Rbk = [ cos(pi) sin(pi) 0;...
        -sin(pi) cos(pi) 0; ...
        0       0      1];
    R = diag([1 1 1]);
else
    xT = initial.xT;
    yT = initial.yT;
    zT = initial.zT;
    wT = initial.wT;
    cT = initial.cT;
    R = initial.R;
end

% find stream max
smax = 0;
stkPtr = 1;
cmdT = zeros(length(str),1);
for i=1:length(str)
    cmdT(i) = str{i}.name;
    switch cmdT(i)
        case '[' %push the stack
            stkPtr = stkPtr +1 ;
            if (smax<stkPtr)
                smax=stkPtr;
            end
        case ']' %pop the stack
            stkPtr = stkPtr +1 ;
    end
end

vC = 1;  %  vertex counter
stC = 1; %  stream counter
nofST = 1;
c=0;

colors{stC}  = cT;
widths{stC}(vC) = wT;
lines{stC}(vC,:) = [xT, yT, zT];
times{stC}(vC) = 0;
vC=vC+1;

for i=1:length(str)

    switch cmdT(i)
        case '#'
            wT = str{i}.width;
            if (vC>1)
                widths{stC}(vC-1) =wT;
            end
        case 'C'
            cT = str{i}.color;
            if (vC>1)
                colors{stC} =cT;
            end
        case 'F'
            lenF = str{i}.length;
            if isfield(str{i},'time')
                t = str{i}.time;
            else
                t=NaN;
            end
            newxT = xT + lenF*R(1,1);
            newyT = yT + lenF*R(2,1);
            newzT = zT + lenF*R(3,1);
            lines{stC}(vC,:) = [newxT newyT newzT];
            widths{stC}(vC) =wT;
            times{stC}(vC) = t;
            colors{stC}(:) = cT;
            vC = vC + 1;
            xT = newxT;
            yT = newyT;
            zT = newzT;
            c=c+1;
        case '+'
            a = str{i}.delta;
            Rup = [    cos(a) sin(a) 0;...
                -sin(a) cos(a) 0; ...
                0       0      1];
            R2 = R * Rup;
            R = R2;
        case '-'
            a = str{i}.delta;
            Rum = [    cos(-a) sin(-a) 0;...
                -sin(-a) cos(-a) 0; ...
                0       0      1];
            R2 = R * Rum;
            R = R2;
        case '&'
            a = str{i}.delta;
            Rlp = [ cos(a) 0  -sin(a) ;...
                0          1     0; ...
                sin(a)    0   cos(a)];
            R2 = R * Rlp;
            R = R2;
        case '^'
            a = str{i}.delta;
            Rlm = [    cos(-a) 0  -sin(-a) ;...
                0          1     0; ...
                sin(-a)    0   cos(-a)];
            R2 = R * Rlm;
            R = R2;
        case '\'
            a = str{i}.delta;
            Rhp = [  1     0     0; ...
                0 cos(a) -sin(a) ;...
                0 sin(a)  cos(a) ] ;
            R2 = R * Rhp;
            R = R2;
        case '/'
            a = str{i}.delta;
            Rhm = [  1     0     0; ...
                0 cos(-a) -sin(-a) ;...
                0 sin(-a)  cos(-a) ] ;
            R2 = R * Rhm;
            R = R2;
        case '|'
            R2 = R * Rbk;
            R = R2;
        case '[' %push the stack
            stack(stkPtr).xT = xT ;
            stack(stkPtr).yT = yT ;
            stack(stkPtr).zT = zT ;
            stack(stkPtr).wT = wT;
            stack(stkPtr).cT = cT;
            stack(stkPtr).R = R;
            stack(stkPtr).vC = vC;
            stack(stkPtr).stC = stC;
            stkPtr = stkPtr +1 ;

            stC  = nofST + 1;
            nofST = nofST +1;
            vC = 1;
            lines{stC}(vC,:) = [xT, yT, zT];
            widths{stC}(vC) =wT;
            colors{stC} = cT;
            vC = vC +1;

        case ']' %pop the stack
            if (vC>1)
                widths{stC}(vC-1) = wT;
            end
            %
            stkPtr = stkPtr -1;
            xT = stack(stkPtr).xT ;
            yT = stack(stkPtr).yT ;
            zT = stack(stkPtr).zT ;
            wT = stack(stkPtr).wT;
            cT = stack(stkPtr).cT;
            R = stack(stkPtr).R;
            vC= stack(stkPtr).vC;
            stC = stack(stkPtr).stC;
    end

end









