function new = ruleCreateRoot(x, r, rs, la, las, lb, lbs, ln, lns, nob, nobs, ...
    theta, thetas, a, as, color, branch, deflect, dx, dt, lifetime, userlambda)
% ruleCreateRoot: Production rule that creates a root.
%
% Parameters:
% x         :letter that is replaced by the production rule
% r         :initial growth speed (cm/day)
% rs        :standard deviation of initial growth speed
% la        :length of apical zone (cm)
% las       :standard deviation of apical zone (cm)
% lb        :length of basal zone (cm)
% lbs       :standard deviation of basal zone (cm)
% ln        :spacing between branches (cm)
% lns       :standard deviation of spacing (equal for the whole branch) (cm)
% nob       :number of branches
% nobs      :deviation of number of branches
% theta     :angle betwenn branch and preceeding branch
% thetas    :standard deviation of angle
% a         :radius (cm)
% as        :standard deviation of radius
% color     :color ([r g b])
% branch    :initial string containing a branch
% deflect   :production rule for root tip deflection
% dx        :spatial discretization along the root (cm)
% dt        :time step
% (lifetime):time the root lives. Default lifetime=inf
% (userlambda): user defined growth fucntion (default=negative exponential)
%
% new       :resulting string
%
% See also: ruleDefinition, ruleCreateHighestOrderRoot, ruleBranch
%
% Copyright 2008-2009 Daniel Leitner. See license.txt for details.
%

r = r + randn*rs; % apply standard deviation s
la = la + randn*las;
lb = lb + randn*lbs;
ln = ln + randn*lns;
nob = nob + round(randn*nobs);
theta = theta + randn*thetas;
a = a + randn*as;
la = max(la,0); % check if numbers are non-negative
lb = max(lb,0);
ln = max(ln,0);
nob = max(nob,0);
theta = max(theta,0);
a = max(a, 0.0001);
k = la+lb+ln*nob; % total length of root

if (nargin<21)
    lifetime=inf;
end

lambda0 = @(t,y) k*(1-exp(-(r/k)*t));
if (nargin<22)
    lambda = lambda0;
else
    lambda = userlambda;
end

if r>0 && k>0
    
    rules = getTurtle;
    
    tau = @(l) -k/r*log((k-l)/k);

    %apical zone
    if la>dx/10
        apical.name = 'G';
        apical.t = 0;
        apical.l = 0;
        t0 = tau(k-la);
        tend = tau(k-dx/10);
        apical.apply = @(x,dt) ruleSectionGrowth(x,lambda,lambda0(t0,apical),lambda0(tend,apical),...
            t0, tend, rules.TurnAround,deflect,dx,dt);
    else
        apical = rules.TurnAround;
    end

    %branching zone
    if (nob>0)
        branching.name='G';
        branching.c=0;

        times = 1:nob;
        delays = 1:nob;
        for i=1:nob
            l = lb + ln*(i-1);
            times(i) = tau(l);
            delays(i) = tau(l+la);
        end
        delays = delays - times;
        times(nob+1) = tau(lb+ln*nob);

        branching.apply = @(x,dt) ruleBranch(x,lambda,times,delays,branch,apical,deflect,dx,dt);
    else
        branching=apical;
    end

    %basal zone
    if lb>0
        basal.name = 'G';
        basal.t = 0;
        basal.l = 0;
        t0=0;
        tend = tau(lb);
        basal.apply = @(x,dt) ruleSectionGrowth(x,lambda,lambda0(t0,basal),lambda0(tend,basal),...
            t0, tend, branching,deflect,dx,dt);
    else
        basal=branching;
    end
    
    if isfield(x,'xT')
        basal.xT = x.xT;
        basal.yT = x.yT;
        basal.zT = x.zT;
        basal.wT = x.wT;
        basal.cT = x.cT;
        basal.R = x.R;
    else
        disp('ruleCreateRoot: no label');
    end
    
    %Create the root.
    die = {};
    if (lifetime~=inf)
        cdie = rules.Color;
        cdie.color = [1,1,1];
        die.name='d';
        die.apply = @(x,dt) ruleDelay(x,lifetime,cdie,dt);
    end
    rules.LW.width=2*a;
    rules.BranchRotate.delta = theta;
    rules.Color.color=color;
    br = rules.BranchRotate.apply(rules.BranchRotate);
    new = [{rules.Push}, {rules.LW}, {rules.Color}, die, br];
    basal = labelTip(basal, new);

    %apply time overhead
    branch = basal.apply(basal,dt);

    new = [new, branch, {rules.Pop}];

else

    new ={};

end
